"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModuleProxy = exports.createModuleProxy = void 0;
const tslib_1 = require("tslib");
const js_magic_1 = require("js-magic");
const values = require("lodash/values");
const define_1 = require("@hyurl/utils/define");
const util_1 = require("../util");
function createModuleProxy(name) {
    let proxy = function (...args) {
        if (new.target) {
            return proxy.create(...args);
        }
        else {
            return proxy.instance(args[0]);
        }
    };
    Object.setPrototypeOf(proxy, ModuleProxy.prototype);
    define_1.default(proxy, "name", name);
    define_1.default(proxy, "remoteSingletons", util_1.dict());
    define_1.default(proxy, "children", util_1.dict());
    proxy[Symbol.toStringTag] = "ModuleProxy";
    proxy[Symbol.hasInstance] = function ModuleProxy(_) {
        return false;
    };
    return js_magic_1.applyMagic(proxy, true);
}
exports.createModuleProxy = createModuleProxy;
let ModuleProxy = class ModuleProxy {
    get path() {
        return void 0;
    }
    get loader() {
        return null;
    }
    get singletons() {
        return null;
    }
    get exports() {
        return null;
    }
    get proto() {
        return null;
    }
    get ctor() {
        return null;
    }
    create(..._) {
        return this.instance(util_1.local);
    }
    instance(route = util_1.local) {
        if (route === util_1.local) {
            throw new TypeError("Local instance is not supported by standalone client");
        }
        if (typeof route === "string" && this.remoteSingletons[route]) {
            return this.remoteSingletons[route];
        }
        let singletons = values(this.remoteSingletons)
            .filter(item => item[util_1.readyState] === 1);
        let count = singletons.length;
        if (count === 0) {
            util_1.throwUnavailableError(this.name);
        }
        else if (count === 1) {
            return singletons[0];
        }
        else {
            let id = util_1.evalRouteId(route);
            return singletons[id % count];
        }
    }
    fallbackToLocal(enable = void 0) {
        if (enable === void 0) {
            return false;
        }
        else if (enable === true) {
            this.instance(util_1.local);
            return this;
        }
    }
    __get(prop) {
        if (prop in this) {
            return this[prop];
        }
        else if (prop in this.children) {
            return this.children[prop];
        }
        else if (typeof prop != "symbol") {
            return this.children[prop] = createModuleProxy(this.name + "." + String(prop));
        }
    }
    __has(prop) {
        return (prop in this) || (prop in this.children);
    }
    toString() {
        return this.name;
    }
    toJSON() {
        return this.name;
    }
};
ModuleProxy = tslib_1.__decorate([
    js_magic_1.applyMagic
], ModuleProxy);
exports.ModuleProxy = ModuleProxy;
//# sourceMappingURL=proxy.js.map