"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcChannel = exports.RpcEvents = void 0;
const util_1 = require("../util");
const bsp_1 = require("bsp");
var RpcEvents;
(function (RpcEvents) {
    RpcEvents[RpcEvents["HANDSHAKE"] = 0] = "HANDSHAKE";
    RpcEvents[RpcEvents["CONNECT"] = 1] = "CONNECT";
    RpcEvents[RpcEvents["BROADCAST"] = 2] = "BROADCAST";
    RpcEvents[RpcEvents["INVOKE"] = 3] = "INVOKE";
    RpcEvents[RpcEvents["RETURN"] = 4] = "RETURN";
    RpcEvents[RpcEvents["YIELD"] = 5] = "YIELD";
    RpcEvents[RpcEvents["THROW"] = 6] = "THROW";
    RpcEvents[RpcEvents["PING"] = 7] = "PING";
    RpcEvents[RpcEvents["PONG"] = 8] = "PONG";
})(RpcEvents = exports.RpcEvents || (exports.RpcEvents = {}));
class RpcChannel {
    constructor(options, host) {
        this.host = "0.0.0.0";
        this.port = 9000;
        this.path = "";
        if (typeof options === "object") {
            Object.assign(this, options);
        }
        else if (typeof options === "number") {
            Object.assign(this, { host, port: options });
        }
        else {
            this.path = util_1.absPath(options);
        }
        this.codec || (this.codec = "CLONE");
        this.bsp = new bsp_1.BSP(util_1.getCodecOptions(this.codec));
    }
    get dsn() {
        let dsn = this.path ? "ipc://" : "rpc://";
        if (this.path) {
            dsn += this.path;
        }
        else if (this.port) {
            if (this.host) {
                dsn += this.host + ":";
            }
            dsn += this.port;
        }
        return dsn;
    }
    onError(handler) {
        this.errorHandler = handler;
    }
    static registerError(ctor) {
        global[ctor.name] = ctor;
    }
}
exports.RpcChannel = RpcChannel;
//# sourceMappingURL=channel.js.map